package modele;

import modele.CouleurWagon;
import modele.Joueur;
import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public class TestUtils {
    /**
     * Renvoie un attribut d'un objet  partir de son nom.
     * La mthode cherche s'il existe un champ dclar dans la classe de l'objet et
     * si ce n'est pas le cas remonte dans la hirarchie des classes jusqu' trouver
     * un champ avec le nom voulu ou renvoie null.
     * 
     * @param obj  objet dont on cherche le champ
     * @param name nom du champ
     * @return le champ de l'objet, avec un type statique Object
     */
    public static Object getAttribute(Object obj, String name) {
        Class c = obj.getClass();
        while (c != null) {
            try {
                Field field = c.getDeclaredField(name);
                field.setAccessible(true);
                return field.get(obj);
            } catch (NoSuchFieldException e) {
                c = c.getSuperclass();
                continue;
            } catch (IllegalAccessException e) {
                e.printStackTrace();
                return null;
            }
        }
        return null;
    }

    /**
     * Met les cartes wagon passes en argument dans la main d'un joueur (la main
     * est vide avant, pour contenir exactement les cartes indiques)
     * 
     * @param joueur le joueur dont on fixe les cartes en main
     */
    public static void setCartesWagon(Joueur joueur, CouleurWagon... cartesWagon) {
        while (!joueur.getCartesWagon().isEmpty()) {
            joueur.getCartesWagon().remove(0);
        }
        Collections.addAll(joueur.getCartesWagon(), cartesWagon);
    }

    /**
     * Teste si la liste de CouleurWagon passe en premier argument contient
     * exactement les couleurs indiques, indpendamment de leur ordre
     * 
     * @param liste    la liste de CouleurWagon  tester
     * @param couleurs les couleurs que la liste devrait contenir (avec leur
     *                 multiplicit)
     * @return true si la liste contient exactement les couleurs indiques, avec
     *         leur multiplicit, false sinon
     */
    public static boolean contientExactement(List<CouleurWagon> liste, CouleurWagon... couleurs) {
        if (liste.size() != couleurs.length) {
            return false;
        }
        Arrays.sort(couleurs);
        Collections.sort(liste);

        for (int i = 0; i < couleurs.length; i++) {
            if (couleurs[i] != liste.get(i)) {
                return false;
            }
        }
        return true;
    }
}
